// FRAM-Treiber   Dr. Weisser 15.3.2019     24.10.2019
//***************************************************************************
//  File........: fm24c64.c
//  Target(s)...: ATmega328p, mega2560, ATmega32u4 
//  Compiler....: AVR-GCC
//  Description.: Routinen fuer FRAM FM24C64B (8k x 8byte =64kbyte) 
//                Vorsicht: der FRAM-Baustein benoetigt 4.5-5.5V 
//                auch nutzbar fuer FM24W256 (32k x 8byte =256kbyte) 
//***************************************************************************
/*  15.3.2019 Uebernahme Code von hdc1080.c, sram23lcv.c 
              Code gekuerzt, umgeschrieben 
    16.3.2019 Programm uebersetzt fehlerfrei 

*/

#include <stdint.h>       // Integer-Datentypen 8-64bit
#include <avr/io.h>       // Pins und Interrupt-Tabelle -> siehe auch iom328p.h
#include "main.h"         // CPU-Frequenz, sbi, cbi
#include "delay.h"        // Wartezeiten 2us, ms
#include "fm24c64.h"      // FRAM-Pins, Funktionen + Prototypen
#include "i2c.h"          // I2C-Zugriffe
#include "usart.h"        // USART-Funktionen fuer Debug-Ausgaben
#include "out.h"          // Ausgaben USART, LCD
#include <avr/pgmspace.h> // PSTR

#define FM24C64  1    // 1: FM24C64   0: kein FM24C64 siehe fm24c64.h

#define DEBUG    1    // 0: keine Debug-Ausgaben 1: Debug-Ausgaben

// defines fuer I2C-Pins stehen in i2c.c
// defines fuer I2C-Adressen Read/Write stehen in fm24c64.h



#if FM24C64 //
#endif // FM24C64



void fram_adr(unsigned int adr) // FRAM Schreiben/Lesen ab Adresse 
                                // -> gibt Slave-Adresse + Adresse 16bit aus 
                                // FRAM ueber I2C ansprechen 
                                // i2c_stop() muss am Ende des Schreibens folgen 
{
   unsigned int temp;           // Hilfsvariable 
   unsigned char adr_h, adr_l;  // Speicheradresse 16bit in 2 bytes teilen 
   
   // Bytes schreiben ab Adresse adr (16bit) 
   // Adresse zerlegen in high und low 
   temp=adr;                          // Hilfsvariable nutzen 
   adr_l= (unsigned char) temp;       // untere 8bit der Adresse 
   temp=adr; 
   adr_h= (unsigned char) (temp >>8); // obere 8bit der 16bit um 8bit rechts schieben
   
   // Schreiben/Lesen vorbereiten 
   i2c_start();            // Start-Zeichen auf I2C-Bus
   
   // Baustein-Adresse Slave auf Bus schicken 
   i2c_write(FRAM_ADR_W);  // FRAM ansprechen fuer Schreiben
   
   // Speicheradresse MSB schreiben 
   i2c_write(adr_h);       // Adresse high byte schicken 
   
   // Speicheradresse LSB schreiben 
   i2c_write(adr_l);       // Adresse low byte schicken 
   
   // nun fertig fuer Schreiben Daten - 1 oder mehr byte 
   // am Ende des Schreibens muss ein i2c_stop() folgen 
   
}



void fram_wr(unsigned char byte) // schreibt byte ins FRAM ab gesetzter Adresse 
                                 // Slave-Adresse + Adresse muss zuvor geschickt sein 
                                 // z.B. mit fram_adr() 
                                 // FRAM ueber I2C ansprechen 
                                 // i2c_stop() muss am Ende des Schreibens folgen
{

   // byte ins FRAM schreiben 
   i2c_write(byte);   // schreibt byte ins FRAM 
   
   // am Ende des Schreibens muss ein i2c_stop() folgen 

}



unsigned char fram_rd(void)      // liest 1 byte vom Slave, ab Adresse 
                                 // gibt dazu Slave-Adresse aus 
{
   unsigned char data; 
   
   // Auslesen vorbereiten 
   i2c_start();            // Start-Zeichen auf I2C-Bus 
   
   // Baustein-Adresse Slave auf Bus schicken 
   i2c_write(FRAM_ADR_R);  // FRAM ansprechen fuer Lesen 
   
   // nur ein byte lesen -> 0 als Argument 
   data=i2c_read(0);       // lesen byte, dann NACK setzen= ACK nicht setzen 
   
   // Lesen abbrechen 
   i2c_stop();             // Stop-Zeichen senden 
   
   return data;  // 
   
}



void fram_test(void)       // FRAM-Test: Schreiben, Lesen 
{
   unsigned int adr, i; 
   unsigned char byte=0;         // Testdaten 

   // Meldung seriell ausgeben 
   outs(USART,PSTR("\nFRAM-Test:")); // Meldung ausgeben
   
   outs(USART,PSTR("\nsequentiell schreiben\n")); // Meldung ausgeben
   

   // Bytes schreiben ab Adresse adr (16bit) 
   adr=0x0000; 
   
   // Startadresse FRAM festlegen fuer Testdaten: Adresse 16bit 
   fram_adr(adr);     // FRAM ab Adresse 0 beschreiben: 16bit 

/*
   // alternativ statt fram_adr(adr); dies mit Einzelbefehlen machen 
   // Adresse zerlegen in high und low 
   temp=adr; 
   adr_l= (unsigned char) temp;       // untere 8bit der Adresse 
   temp=adr; 
   adr_h= (unsigned char) (temp >>8); // obere 8bit der 16bit um 8bit rechts schieben
   
   // Schreiben/Lesen vorbereiten 
   i2c_start();            // Start-Zeichen auf I2C-Bus 
   
   // Baustein-Adresse Slave auf Bus schicken 
   i2c_write(FRAM_ADR_W);  // FRAM ansprechen fuer Schreiben 
   
   // Speicheradresse MSB schreiben 
   i2c_write(adr_h);       // Adresse high byte schicken 
   
   // Speicheradresse LSB schreiben 
   i2c_write(adr_l);       // Adresse low byte schicken 
*/
   
   for (i=0; i<=255; i++){
      // 256 bytes ins FRAM schreiben 
      i2c_write(byte);     // byte ins FRAM schreiben 
                           // oder stattdessen fram_wr(); 
      byte++;              // bytes aufsteigend sequentiell schreiben 
   }
   
   // Schreiben beenden 
   i2c_stop();             // Stop-Zeichen auf I2C-Bus
   
   outs(USART,PSTR("\nFRAM schreiben beendet\n")); // Meldung ausgeben
   
   
   
   
   outs(USART,PSTR("\nFRAM nun testweise auslesen\n")); // Meldung ausgeben 
   
   // FRAM-Adresse fuer Auslesen angeben 
   // Bytes auslesen ab Adresse adr (16bit) 
   adr=0x0000; 
   
   // Startadresse FRAM festlegen fuer Testdaten: Adresse 16bit 
   fram_adr(adr);     // FRAM ab Adresse 0 beschreiben: 16bit 
   
/* 
   // Adresse zerlegen in high und low 
   temp=adr; 
   adr_l= (unsigned char) temp;       // untere 8bit der Adresse 
   temp=adr; 
   adr_h= (unsigned char) (temp >>8); // obere 8bit der 16bit um 8bit rechts schieben
   
   // Schreiben/Lesen vorbereiten 
   i2c_start();            // Start-Zeichen auf I2C-Bus 
   
   // Baustein-Adresse Slave auf Bus schicken 
   i2c_write(FRAM_ADR_W);  // FRAM ansprechen fuer Schreiben 
   
   // Speicheradresse MSB schreiben 
   i2c_write(adr_h);       // Adresse high byte schicken 
   
   // Speicheradresse LSB schreiben 
   i2c_write(adr_l);       // Adresse low byte schicken 
   
*/ 


   // erneut Start geben fuer Auslesen 
   i2c_start();            // Start-Zeichen auf I2C-Bus 
   
   // erneut Baustein-Adresse Slave auf Bus schicken - nun fuer Lesen ! 
   i2c_write(FRAM_ADR_R);  // FRAM ansprechen fuer Lesen 
   
   // 255 bytes aus FRAM auslesen mit (0) 
   for (i=0; i<=254; i++){ 
      byte=i2c_read(0);    // byte aus FRAM lesen 
      
      // Daten testweise seriell ausgeben 
      outs(USART,PSTR("\nbyte: ")); // Meldung ausgeben
      out_u08(USART,byte,0,0);
      outs(USART,PSTR("\n"));       // Meldung ausgeben
   }
   
   // letztes byte auslesen -> mit (1) 
   byte=i2c_read(1);       // letztes byte aus FRAM lesen 
   
   outs(USART,PSTR("\nbyte: ")); // Meldung ausgeben
   out_u08(USART,byte,0,0);
   outs(USART,PSTR("\n"));       // Meldung ausgeben
   
   // Auslesen beendet, Bus freigeben 
   i2c_stop();             // Stop-Zeichen auf I2C-Bus
   
   outs(USART,PSTR("\nFRAM auslesen beendet\n")); // Meldung ausgeben
   

}

