// I2C-Treiber   Dr. Weisser 14.10.2016     30.6.2019
//i2c.h

#ifndef _I2C_H_
#define _I2C_H_

// Hardwarewahl: sollte aus Makefile korrekt kommen -> dann nicht noetig
//#define __AVR_ATmega328P__ 1 // Hardwarewahl: ATmega328p
//#define __AVR_ATmega2560__ 1 // Hardwarewahl: ATmega2560
//#define __AVR_ATmega32U4__ 1 // Hardwarewahl: ATmega32U4..

#define TWI 1        // TWI-Hardware nutzen statt bitbanging, ggf. auch im Makefile setzen

// Pin-Definitionen
// fuer bitbanging beliebige Pins moeglich
// fuer Hardware I2C nur bestimmte Pins moeglich: siehe Datenblatt
#if defined(__AVR_ATmega328P__) // Hardware Atmega328/Arduino nano3

#define SCL_PIN      5  // Pin 5 am Port
#define SCL_PORT     PORTC
#define SCL_INIT()   cbi(DDRC,5); cbi(PORTC,5)  // SCL input, Pegel low (kein Pullup)
#define SCL_L()      sbi(DDRC,5)                // SCL low (durch Schalten auf Ausgabe)
#define SCL_H()      cbi(DDRC,5)                // SCL hi-Z (durch Schalten auf Eingabe)
                                                // pullup am Pin zieht dann nach hi
#define SCL_VAL      bit_is_set(PINC,5)         // SCL level

#define SDA_PIN      4  // Pin 4 am Port
#define SDA_PORT     PORTC
#define SDA_INIT()   cbi(DDRC,4); cbi(PORTC,4)  // SCL input, Pegel low (kein Pullup)
#define SDA_L()      sbi(DDRC,4)                // SDA low (durch Schalten auf Ausgabe)
#define SDA_H()      cbi(DDRC,4)                // SDA hi-Z (durch Schalten auf Eingabe)
                                                // pullup am Pin zieht dann nach hi
#define SDA_VAL      bit_is_set(PINC,4)         // SDA level


#elif defined(__AVR_ATmega2560__) || defined(__AVR_ATmega32U4__) || defined(__AVR_AT90USB1286__) 
                                                // Hardware 2560/32U4/1286
                                                // ggf. Zeile umbrechen mit "\"

#define SCL_PIN      0  // Pin 0 am Port
#define SCL_PORT     PORTD
#define SCL_INIT()   cbi(DDRD,0); cbi(PORTD,0)  // SCL input, Pegel low (kein Pullup)
#define SCL_L()      sbi(DDRD,0)                // SCL low (durch Schalten auf Ausgabe)
#define SCL_H()      cbi(DDRD,0)                // SCL hi-Z (durch Schalten auf Eingabe)
                                                // pullup am Pin zieht dann nach hi
#define SCL_VAL      bit_is_set(PIND,0)         // SCL level

#define SDA_PIN      1  // Pin 1 am Port
#define SDA_PORT     PORTD
#define SDA_INIT()   cbi(DDRD,1); cbi(PORTD,1)  // SCL input, Pegel low (kein Pullup)
#define SDA_L()      sbi(DDRD,1)                // SDA low (durch Schalten auf Ausgabe)
#define SDA_H()      cbi(DDRD,1)                // SDA hi-Z (durch Schalten auf Eingabe)
                                                // pullup am Pin zieht dann nach hi
#define SDA_VAL      bit_is_set(PIND,1)         // SDA level

#endif // Hardwareabfrage 




// TWI General Master status codes
#define TWI_START                   0x08  // START has been transmitted
#define TWI_REP_START               0x10  // Repeated START has been transmitted
#define TWI_ARB_LOST                0x38  // arbitration lost

// TWI Master Transmitter status codes
#define TWI_MTX_ADR_ACK             0x18  // SLA+W has been transmitted and ACK received
#define TWI_MTX_ADR_NACK            0x20  // SLA+W has been transmitted and NACK received
#define TWI_MTX_DATA_ACK            0x28  // data byte has been transmitted and ACK received
#define TWI_MTX_DATA_NACK           0x30  // data byte has been transmitted and NACK received

// TWI Master Receiver status codes
#define TWI_MRX_ADR_ACK             0x40  // SLA+R has been transmitted and ACK received
#define TWI_MRX_ADR_NACK            0x48  // SLA+R has been transmitted and NACK received
#define TWI_MRX_DATA_ACK            0x50  // data byte has been received and ACK transmitted
#define TWI_MRX_DATA_NACK           0x58  // data byte has been received and NACK transmitted

// TWI Slave Transmitter status codes
#define TWI_STX_ADR_ACK             0xA8  // own SLA+R has been received; ACK has been returned
#define TWI_STX_ADR_ACK_M_ARB_LOST  0xB0  // arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
#define TWI_STX_DATA_ACK            0xB8  // data byte in TWDR has been transmitted; ACK has been received
#define TWI_STX_DATA_NACK           0xC0  // data byte in TWDR has been transmitted; NOT ACK has been received
#define TWI_STX_DATA_ACK_LAST_BYTE  0xC8  // last data byte in TWDR has been transmitted (TWEA = 0); ACK has been received

// TWI Slave Receiver status codes
#define TWI_SRX_ADR_ACK             0x60  // own SLA+W has been received ACK has been returned
#define TWI_SRX_ADR_ACK_M_ARB_LOST  0x68  // arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
#define TWI_SRX_GEN_ACK             0x70  // general call address has been received; ACK has been returned
#define TWI_SRX_GEN_ACK_M_ARB_LOST  0x78  // arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_ACK        0x80  // previously addressed with own SLA+W; data has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_NACK       0x88  // previously addressed with own SLA+W; data has been received; NOT ACK has been returned
#define TWI_SRX_GEN_DATA_ACK        0x90  // previously addressed with general call; data has been received; ACK has been returned
#define TWI_SRX_GEN_DATA_NACK       0x98  // previously addressed with general call; data has been received; NOT ACK has been returned
#define TWI_SRX_STOP_RESTART        0xA0  // a STOP condition or repeated START condition has been received while still addressed as Slave

// TWI Miscellaneous status codes
#define TWI_NO_STATE                0xF8  // no relevant state information available; TWINT = 0
#define TWI_BUS_ERROR               0x00  // bus error due to an illegal START or STOP condition



void i2c_init(void);                      // I2C-Buspins init
void i2c_delay(void);                     // delay fuer 100/400kHz bei 16MHz-Clock
void i2c_start(void);                     // Start-Kommando
void i2c_stop(void);                      // Stop-Kommando
unsigned char i2c_write(unsigned char);   // byte schreiben
unsigned char i2c_read(unsigned char);    // byte lesen
unsigned char i2c_read_block(unsigned char,unsigned char,unsigned int,void*);        // blockweise lesen
unsigned char i2c_write_block(unsigned char,unsigned char,unsigned int,const void*); // blockweise schreiben


#endif
