// FM24C64-FRAM-Projekt   Dr. Weisser 16.3.2019  24.10.2019
//***************************************************************************
//  File........: main.c
//  Target(s)...: ATmega2560 -> ATmega32u4
//  Compiler....: AVR-GCC
//  Description.: FM24C64 (bzw. FM24W256) an ATmega32u4 
//                übersetzt mit Makefile: make clean, make all
//                per USB auf ATmega32u4: make program
//***************************************************************************
/*  16.3.2019 Pins fuer FRAM definiert in fm24c64.h und main.c 
              Makefile angepasst 
              Funktionen fuer Schreiben, Lesen FRAM 
              Programm uebersetzt fehlerfrei 
              serielle Verbindung klappt - COM10 256kbd 
              serielle Ausgaben kommen 
              FRAM lesen klappt nicht da kein FRAM angeschlossen 
              Funktion fram_test(); erstellt 
   24.10.2019 umgestellt auf ATmega32u4 
              outs(USART,PSTR) -> outs_p(USART,PSTR) 
              unsinnige USART-Ausgaben. Warum?

*/

#include <stdint.h>        // Integer-Datentypen 8-64bit 
#include <avr/io.h>        // Pins und Interrupt-Tabelle -> siehe auch iom328p.h 
#include <avr/interrupt.h> // Interrupts 
#include "main.h"          // CPU-Frequenz, sbi, cbi 
#include "usart.h"         // USART-Funktionen 
#include "out.h"           // Ausgabe-Funktionen Zahlen, USART 
#include "i2c.h"           // I2C-Zugriffe 
#include "fm24c64.h"       // FRAM-Funktionen 
#include "delay.h"         // Delay-Funktionen us, ms 
#include <avr/pgmspace.h>  // PSTR 

#define DEBUG 0 // Debug-Ausgaben einschalten: 1 oder ausschalten: 0 

/* fuer Timing-Tests */
#if DEBUG 
#endif // DEBUG 

// Global variables 

// Private functions 
//

/*****************************************************************************
*   Function name : main
*   Returns :       none
*   Parameters :    none
*   Purpose :       contains main loop of the program
*****************************************************************************/

int main(void)
{
   unsigned int adr, temp ,i; 
   unsigned char adr_h, adr_l;   // Speicheradresse 16bit 
   unsigned char byte=0;         // Testdaten 
   
   // Initalisierung CPU, Pins in/out, SRAM, USART 
   initialization();  // init CPU, Pins in/out, USART 
   
   //sei();  // Interrupts freigeben 
   
   //fram_test();  // FRAM-Test als Funktion aufrufen 
   
   outs_p(USART,PSTR("\nteste FRAM")); // Meldung ausgeben 
   //delay_us(100); 
   
   outs_p(USART,PSTR("\nFRAM sequentiell schreiben\n")); // Meldung ausgeben 
   
   while(1){
      outs_p(USART,PSTR("\nteste FRAM")); // Meldung ausgeben   
      outs_p(USART,PSTR("\nFRAM sequentiell schreiben\n")); // Meldung ausgeben 
   }
   
   return 1;
}



/*****************************************************************************
*   Function name : initialization
*   Returns :       none
*   Parameters :    none
*   Purpose :       initialize different modules
*****************************************************************************/
void initialization(void)
{

   CLKPR = (1<<CLKPCE);   // set Clock Prescaler Change Enable
                             // Bitmuster von 1 wird um CLKPCE=7 Stellen nach links geschoben
                             // und dann ins Register CLKPR geschrieben. 1000000 also 

   // Prescaler = 1 -> 16 MHz bei 16 MHz Quarz
   CLKPR = (0<<CLKPS1) | (0<<CLKPS0);   // laeuft mit 16 MHz
                         
   //CPU_PRESCALE(0);  // siehe #define in main.h: (CLKPR = 0x80, CLKPR = (n))
                     // erst 1000 0000, dann 0000 0000 fuer ClockDivFactor 1 -> 16 MHz
                     // vgl. Datenblatt ATmega16U4/32U4 2016 S.39 

   // Disable Analog Comparator (power save)
   ACSR = (1<<ACD);      // bit 7 = ACD im Analog Comp Status Register auf 1 setzen schaltet aus
                         // Bitmuster von 1 wird ACD=7 Stellen nach links geschoben. 1000000 also.
                         // spart Strom weil Analog Comp aus.

   // Disable Digital input on analog inputs (power save)
   // DIDR0 = (1<<ADC5D) | (1<<ADC4D) | (1<<ADC3D) | (1<<ADC2D) | (1<<ADC1D) | (1<<ADC0D); //
                         // Digital Input Disable Register 0
                         // schaltet digitale Eingangspuffer der als ADC benutzten Eingänge ab
                         // um Strom zu sparen -> kein digital Input dort

   // Disable Digital input on Ain0 und Ain1 (power save)
   // DIDR1 = (0<<AIN1D) | (0<<AIN0D); // Dig. Inputs werden gebraucht, also nicht Buffer abschalten
                         // Digital Input Disable Register 1

   // DDRB Pins teilweise auf Eingabe
   DDRB = (0<<DDB7) | (1<<DDB6) | (1<<DDB5) | (0<<DDB4) | (0<<DDB3) | (1<<DDB2) | (1<<DDB1) | (1<<DDB0);
            // PB7 = D11                            - in
            // PB6 = D10      CS ADC                - out
            // PB5 = D9/A8    CS SRAM               - out
            // PB4 = D8                             - in
            // PB3 = D14 MISO ADC/SRAM              - in
            // PB2 = D16 MOSI ADC/DAC/SRAM          - out
            // PB1 = D15 SCK  ADC/DAC/SRAM          - out
            // PB0 = D17 SS (gibt SPI frei) LED     - out

   // PORTB pullup enable fuer Eingänge
   PORTB = (1<<PB7) | (1<<PB6) | (1<<PB5) | (1<<PB4) | (1<<PB3) | (0<<PB2) | (0<<PB1) | (0<<PB0);
            // PortB Daten Register: Enable pullup
            // damit sind bei PortB-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRC Pins teilweise auf Eingabe: kein PC0,PC1,PC2,PC3,PC4,PC5
   DDRC = (0<<DDC7) | (1<<DDC6);
            // PC7 = D13                            - in
            // PC6 = D5       LDAC DAC              - out
            // PC5 = nicht da                       - in
            // PC4 = nicht da                       - in
            // PC3 = nicht da                       - in
            // PC2 = nicht da                       - in
            // PC1 = nicht da                       - in
            // PC0 = nicht da                       - in

   // PORTC pullup enable fuer Eingänge: kein PC0,PC1,PC2,PC3,PC4,PC5
   PORTC = (1<<PC7) | (1<<PC6);
            // PortC Daten Register: Enable pullup
            // damit sind bei PortC-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRD Pins teilweise auf Eingabe
   DDRD = (0<<DDD7) | (0<<DDD6) | (1<<DDD5) | (0<<DDD4) | (1<<DDD3) | (0<<DDD2) | (0<<DDD1) | (0<<DDD0);
            // PD7 = D6/A7                          - in
            // PD6 = D12/A10                        - in
            // PD5 = TX LED                         - out
            // PD4 = D4/A6                          - in
            // PD3 = D1 TXD1                        - out
            // PD2 = D0 RXD1                        - in
            // PD1 = D2 SDA   FRAM/RTC/LCD          - in
            // PD0 = D3 SCL   FRAM/RTC/LCD          - in

   // PORTD pullup enable fuer Eingänge
   PORTD = (1<<PD7) | (1<<PD6) | (0<<PD5) | (1<<PD4) | (0<<PD3) | (1<<PD2) | (1<<PD1) | (1<<PD0);
            // PortD Daten Register: Enable pullup
            // damit sind bei PortD-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRE Pins teilweise auf Eingabe: kein PE0,PE1,PE3,PE4,PE5,PE7
   DDRE = (1<<DDE6) | (0<<DDE2);
            // PE7 = nicht da                       - in
            // PE6 = D7       CS DAC                - out
            // PE5 = nicht da                       - in
            // PE4 = nicht da                       - in
            // PE3 = nicht da                       - in
            // PE2/HWB = fest auf Masse             - in
            // PE1 = nicht da                       - in
            // PE0 = nicht da                       - in

   // PORTE pullup enable fuer Eingänge: kein PE0,PE1,PE3,PE4,PE5,PE7
   PORTE = (1<<PE6) | (1<<PE2);
            // PortE Daten Register: Enable pullup
            // damit sind bei PortE-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   // DDRF Pins teilweise auf Eingabe: kein PF2,PF3
   DDRF = (0<<DDF7) | (0<<DDF6) | (0<<DDF5) | (0<<DDF4) | (0<<DDF1) | (0<<DDF0);
            // PF7 = A0                             - in
            // PF6 = A1                             - in
            // PF5 = A2                             - in
            // PF4 = A3                             - in
            // PF3 = nicht da                       - in
            // PF2 = nicht da                       - in
            // PF1 = A4                             - in
            // PF0 = A5                             - in

   // PORTF pullup enable fuer Eingänge: kein PF2,PF3
   PORTF = (1<<PF7) | (1<<PF6) | (0<<PF5) | (1<<PF4) | (1<<PF1) | (1<<PF0);
            // PortF Daten Register: Enable pullup
            // damit sind bei PortF-Input-Pins die pullups aktiv
            // unbeschaltete Input-Pins nach high gezogen ->
            // verursachen so keine erhöhte Stromaufnahme,
            // die auftreten würde wenn Input-Pins floaten

   i2c_init();  // I2C-Pins SCL, SDA festlegen 

   //Timer0_init();    // for 10ms Timer

    // initialisiere UART lt. Table 19-12. Datenblatt ATmega328P S.203
            // bei 16MHz U2X0=1 38.4k -> 51 Fehler 0.2%
            //                  57.6k    34        0.8%
            //                  115.2k   16        2.1%
            //                  250k      7        0%
            //                  0.5M      3        0%
            //                  1M        1        0%
   //usart_init(51);  // serielle Schnittstelle init mit 38.4kbd, kein FIFO
   //usart_init(16);  // serielle Schnittstelle init mit 115.2kbd, kein FIFO
   usart_init(7);  // serielle Schnittstelle init mit 256kbd, kein FIFO

   outc(USART,'A');   // Zeichen zur Bestaetigung ruecksenden - klappt
   outc(USART,'\n');  // neue Zeile - klappt auch
   //outc(USB,'A');   // Zeichen zur Bestaetigung ueber USB ruecksenden - nichts zu sehen
   //outc(USB,'\n');  // neue Zeile ueber USB ruecksenden - nichts zu sehen

   //spi_init(); 
   //usb_init();        // USB init: siehe usb_serial.h 

}
